"""Check tests are not skipped in every ci job"""

from __future__ import print_function

import os
import sys
import xml.etree.ElementTree as ET

base_dir = sys.argv[1]

# dict {test: result} where result is False if the test was skipped in every
# job and True otherwise.
always_skipped = {}

for name in os.listdir(base_dir):
    # all test result files are in base_dir/test_result_*/ dirs
    if name.startswith("test_result_"):
        print(f"> processing test result from job {name.replace('test_result_', '')}")
        print("  > tests skipped:")
        result_file = os.path.join(base_dir, name, "test_result.xml")
        root = ET.parse(result_file).getroot()

        # All tests are identified by the xml tag testcase.
        for test in root.iter("testcase"):
            test_name = test.attrib["name"]
            skipped = any(child.tag == "skipped" for child in test)
            if skipped:
                print("    -", test_name)
            if test_name in always_skipped:
                always_skipped[test_name] &= skipped
            else:
                always_skipped[test_name] = skipped

print("\n------------------------------------------------------------------\n")

# List of tests that we don't want to fail the CI if they are skipped in
# every job. This is useful for tests that depend on specific versions of
# numpy or scipy and we don't want to pin old versions of these libraries.
SAFE_SKIPPED_TESTS = ["test_multiple_shipped_openblas"]

fail = False
for test, skipped in always_skipped.items():
    if skipped:
        if test in SAFE_SKIPPED_TESTS:
            print(test, "was skipped in every job but it's fine to skip it")
        else:
            fail = True
            print(test, "was skipped in every job")

if fail:
    sys.exit(1)



import os
from setuptools import Extension, setup
from Cython.Build import cythonize

from build_utils import set_cc_variables
from build_utils import get_openmp_flag

original_environ = os.environ.copy()
try:
    set_cc_variables("CC_OUTER_LOOP")
    openmp_flag = get_openmp_flag()

    use_custom_blas = os.getenv("INSTALL_BLAS", False)
    libraries = [use_custom_blas] if use_custom_blas else []
    custom_suffix = "_custom" if use_custom_blas else ""
    filename = f"nested_prange_blas{custom_suffix}.pyx"

    ext_modules = [
        Extension(
            "nested_prange_blas",
            [filename],
            extra_compile_args=openmp_flag,
            extra_link_args=openmp_flag,
            libraries=libraries,
        )
    ]

    setup(
        name="_openmp_test_helper_nested_prange_blas",
        ext_modules=cythonize(
            ext_modules,
            compiler_directives={
                "language_level": 3,
                "boundscheck": False,
                "wraparound": False,
            },
        ),
    )

finally:
    os.environ.update(original_environ)



import ctypes
from pathlib import Path

from threadpoolctl import LibController, register

path = Path(__file__).parent / "my_threaded_lib.so"
ctypes.CDLL(path)


class MyThreadedLibController(LibController):
    # names for threadpoolctl's context filtering
    user_api = "my_threaded_lib"
    internal_api = "my_threaded_lib"

    # Patterns to identify the name of the linked library to load.
    # If a dynamic library with a matching filename is linked to the python
    # process, it will be loaded as the `dynlib` attribute of the LibController
    # instance.
    filename_prefixes = ("my_threaded_lib",)

    # (Optional) Symbols that the linked library is expected to expose. It is used along
    # with the `filename_prefixes` to make sure that the correct library is identified.
    check_symbols = (
        "mylib_get_num_threads",
        "mylib_set_num_threads",
        "mylib_get_version",
    )

    def get_num_threads(self):
        # This function should return the current maximum number of threads,
        # which is reported as "num_threads" by `ThreadpoolController.info`.
        return getattr(self.dynlib, "mylib_get_num_threads")()

    def set_num_threads(self, num_threads):
        # This function limits the maximum number of threads,
        # when `ThreadpoolController.limit` is called.
        getattr(self.dynlib, "mylib_set_num_threads")(num_threads)

    def get_version(self):
        # This function returns the version of the linked library if it is exposed,
        # which is reported as "version" by `ThreadpoolController.info`.
        get_version = getattr(self.dynlib, "mylib_get_version")
        get_version.restype = ctypes.c_char_p
        return get_version().decode("utf-8")

    def set_additional_attributes(self):
        # This function is called during the initialization of the LibController.
        # Additional information meant to be exposed by `ThreadpoolController.info`
        # should be set here as attributes of the LibController instance.
        self.some_attr = "some_value"


register(MyThreadedLibController)



cimport openmp
from cython.parallel import parallel, prange

import numpy as np

cdef extern from 'cblas.h' nogil:
    ctypedef enum CBLAS_ORDER:
        CblasRowMajor=101
        CblasColMajor=102
    ctypedef enum CBLAS_TRANSPOSE:
        CblasNoTrans=111
        CblasTrans=112
        CblasConjTrans=113
    void dgemm 'cblas_dgemm' (
        CBLAS_ORDER Order, CBLAS_TRANSPOSE TransA,
        CBLAS_TRANSPOSE TransB, int M, int N,
        int K, double alpha, double *A, int lda,
        double *B, int ldb, double beta, double *C, int ldc)

from threadpoolctl import ThreadpoolController


def check_nested_prange_blas(double[:, ::1] A, double[:, ::1] B, int nthreads):
    """Run multithreaded BLAS calls within OpenMP parallel loop"""
    cdef:
        int m = A.shape[0]
        int n = B.shape[0]
        int k = A.shape[1]

        double[:, ::1] C = np.empty((m, n))
        int n_chunks = 100
        int chunk_size = A.shape[0] // n_chunks

        double alpha = 1.0
        double beta = 0.0

        int i
        int prange_num_threads
        int *prange_num_threads_ptr = &prange_num_threads

    inner_info = [None]

    with nogil, parallel(num_threads=nthreads):
        if openmp.omp_get_thread_num() == 0:
            with gil:
                inner_info[0] = ThreadpoolController().info()

            prange_num_threads_ptr[0] = openmp.omp_get_num_threads()

        for i in prange(n_chunks):
            dgemm(CblasRowMajor, CblasNoTrans, CblasTrans,
            chunk_size, n, k, alpha, &A[i * chunk_size, 0], k,
            &B[0, 0], k, beta, &C[i * chunk_size, 0], n)

    return np.asarray(C), prange_num_threads, inner_info[0]
